//
//  Shader.vsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

attribute vec4 position;
attribute vec3 normal;

varying lowp vec4 colorVarying;

uniform mat4 mMatrix;
uniform mat4 vpMatrix;
uniform mat3 normalMatrix;
uniform mat4 viewInverse;
uniform vec4 color;

void main()
{
    vec3 normalDirection = normalize(normalMatrix * normal);
    vec3 lightDirection = normalize(vec3(1.0, -2.0, 2.0));
    vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - mMatrix * position));
    
    float nDotVP = max(0.0, dot(normalDirection, normalize(lightDirection)));
                 
    vec3 specularReflection;
    if (dot(normalDirection, lightDirection) < 0.0) // light source on the wrong side?
    {
        specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
    } else { // light source on the right side

        specularReflection = vec3(1,1,1) * vec3(1,1,1) * pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), 2.0);
    }

    colorVarying.xyz = color.xyz * nDotVP + specularReflection;

    //colorVarying.xyz = normal;
    //colorVarying.xyz = normal;
    
    gl_Position = vpMatrix * mMatrix * position;
}
