//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

varying lowp vec2 frag_uv1;
varying lowp vec4 v_position;
varying lowp vec3 v_normal;
varying lowp float v_fog;
varying lowp vec4 colorVarying;
VARYING_MAT3(tangentSpace);
varying highp vec4 shadowCoord;

uniform sampler2D texture;
uniform sampler2D normalMap;
uniform SHADOW_SAMPLER_FD shadowMap;
uniform sampler2D gloss_map;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp vec3 light_dir;

uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;

F_FOG_UNIFORMS


void main()
{

//    lowp float pixel_size = 1.0/2048.0;
    lowp vec3 lightColor = vec3(1.0, 0.97, 0.9);
    lowp vec4 result = texture2D(texture, frag_uv1);
    lowp vec4 spec_gloss = texture2D(gloss_map, frag_uv1);
    lowp float gloss = spec_gloss.y;
    spec_gloss.x = pow(spec_gloss.x, 1.3) * 2.0;
    highp vec3 nm = normalize(texture2D(normalMap, frag_uv1).xyz*2.0 - 1.0).xyz;
    lowp float shadowVal = 0.0;
    highp vec4 m_shadow_coord = shadowCoord;
    shadowVal = CALC_SHADOW_VAL_FD(shadowMap, m_shadow_coord);
    gl_FragColor = result;
    
    
    mediump vec3 normalDirection = normalize(MAKE_MAT3_FROM_VARYING(tangentSpace) * nm);
    lowp vec3 lightDirection = -normalize(light_dir);
    mediump vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - mMatrix * v_position));
    
    lowp float nDotVP = max(0.0, dot(normalDirection, normalize(lightDirection)));
    lowp float fresnel = 1.0 - pow(dot(normalDirection, viewDirection), 1.0);
                 
    lowp vec3 specularReflection;

    specularReflection = lightColor * (0.5 + spec_gloss.x*(5.0 + 6.0*fresnel)) * pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), 1.0 + 40.0*gloss);


    //lowp float refl_amt = gloss*fresnel;
    
    gl_FragColor.xyz = gl_FragColor.xyz*0.8*ambient_mult + vec3(0.9, 0.9, 1.0)*(fresnel*fresnel*0.2 + 0.02)*(1.0 - nDotVP) + (gl_FragColor.xyz + lightColor*0.3)*0.6*(nDotVP*lightColor*(1.0 - gloss) + (specularReflection))*shadowVal;
    gl_FragColor.w = 1.0;
    
//    ADD_FOG(v_fog, fog_color);
//    COLOR_CORRECT;
    
//    gl_FragColor.xyz = result.xyz;
    

}
