//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//


varying lowp vec2 frag_uv1;
varying highp vec4 v_position;
varying lowp float v_fog;
VARYING_MAT3(tangentSpace);
varying highp vec4 shadowCoord;

uniform sampler2D texture;
uniform sampler2D normalMap;
uniform sampler2D shadowMap;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;

F_FOG_UNIFORMS


void main()
{

    gl_FragColor = texture2D(texture, frag_uv1);
    lowp vec3 lightColor = vec3(1.0, 0.93, 0.80);
    
    lowp vec3 nm = normalize(texture2D(normalMap, frag_uv1).xyz*2.0 - 1.0).xyz;
    lowp float shadowVal = texture2DProj(shadowMap, shadowCoord).x;

    
    highp vec3 normalDirection = MAKE_MAT3_FROM_VARYING(tangentSpace) * nm;
    //mediump vec3 normalDirection = normalize(normalMatrix * v_normal);
    lowp vec3 lightDirection = normalize(vec3(-0.3, 1.0, -0.3));
    highp vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - mMatrix * v_position));

    highp float nDotVP = max(0.0, dot(normalDirection, normalize(lightDirection)));
    
    highp vec3 specularReflection = lightColor * pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), 2.0);

    highp vec3 base = gl_FragColor.xyz;
    gl_FragColor.xyz = base *(vec3(0.9, 0.9, 1.0)*0.85*ambient_mult) + base * (lightColor*1.2*nDotVP + specularReflection*1.5)*shadowVal*diffuse_mult;
    gl_FragColor.w = 1.0;

    ADD_FOG(v_fog, fog_color);
    COLOR_CORRECT;
    

    

}
