//
//  Shader.fsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

varying lowp vec2 frag_uv1;
varying lowp vec4 v_position;
varying lowp vec3 v_normal;
VARYING_MAT3(tangentSpace);

uniform sampler2D texture;
uniform sampler2D normal_map;
uniform sampler2D specular_map;

uniform highp mat4 viewInverse;
uniform highp mat4 mMatrix;
uniform highp mat4 vpMatrix;
uniform highp mat3 normalMatrix;

uniform lowp float ambient_mult;
uniform lowp float diffuse_mult;
uniform highp vec3 light_dir;
uniform highp vec4 color;


void main()
{

    mediump vec3 lightColor = vec3(1.0, 0.93, 0.89);
    lowp vec4 result = texture2D(texture, frag_uv1);
    lowp vec3 nm = normalize(texture2D(normal_map, frag_uv1).xyz*2.0 - 1.0).xyz;
    lowp float specular = texture2D(specular_map, frag_uv1).x;
    lowp float shadowVal = 1.0;
    gl_FragColor = result;
    
    
    mediump vec3 normalDirection = MAKE_MAT3_FROM_VARYING(tangentSpace) * nm;
    mediump vec3 lightDirection = normalize(light_dir);
    mediump vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - v_position));
    
    mediump float nDotVP = max(0.0, dot(normalDirection, normalize(lightDirection)));
                 
    mediump vec3 specularReflection;
    if (dot(normalDirection, lightDirection) < 0.0) // light source on the wrong side?
    {
        specularReflection = vec3(0.0, 0.0, 0.0); // no specular reflection
    } else { // light source on the right side

        specularReflection = lightColor * pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), 8.0)*specular*8.0;
    }

    gl_FragColor.xyz = NORMAL_MAP_LIGHTING(lightColor);
    gl_FragColor.w = 1.0;
    gl_FragColor *= color;
    
    

}
