//
//  Shader.vsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

attribute vec4 position;
attribute vec2 uv1;
attribute vec3 normal;

varying highp vec2 frag_uv1;
varying highp float v_diff;
varying highp float v_spec;


uniform mat4 mMatrix;
uniform mat4 vpMatrix;
uniform mat3 normalMatrix;
uniform mat4 viewInverse;

uniform highp float diffuse_mult;
uniform highp float spec_mult;
uniform highp vec3 light_dir;
uniform highp float glossiness;


void main()
{

    frag_uv1 = uv1;

    
    vec4 worldPosition = mMatrix * position;
    gl_Position = vpMatrix * worldPosition;
    
    
    mediump vec3 normalDirection = normalize(normalMatrix * normal);
    mediump vec3 lightDirection = normalize(light_dir);
    mediump vec3 viewDirection = normalize(vec3(viewInverse * vec4(0.0, 0.0, 0.0, 1.0) - worldPosition));
    
    v_diff = max(0.0, dot(normalDirection, normalize(lightDirection)))*diffuse_mult;
    
    mediump vec3 specularReflection;
    if (dot(normalDirection, lightDirection) < 0.0) {
        v_spec = 0.0;
    } else { // light source on the right side
        v_spec = pow(max(0.0, dot(reflect(-lightDirection, normalDirection), viewDirection)), glossiness)*spec_mult;
    }
}
