//
//  Shader.vsh
//  ios
//
//  Created by Adam Berger on 1/28/13.
//  Copyright (c) 2013 Auxbrain, Inc. All rights reserved.
//

attribute vec4 position;
attribute vec2 uv1;
attribute vec3 normal;
attribute vec4 binormal;
attribute vec2 bone;

varying vec2 frag_uv1;
varying lowp float v_fog;
VARYING_MAT3(tangentSpace);
varying highp vec4 v_shadow_coord;
varying lowp vec3 v_bloom_dir;

uniform mat4 vpMatrix;
uniform mat4 viewInverse;

uniform mat4 mMatrix;
uniform mat3 normalMatrix;

uniform mat4 shadow_projection;

UNIFORM_MAT4_ARRAY(mMatrices, 10);
UNIFORM_MAT3_ARRAY(minvMatrices, 10);

uniform lowp float fog_mult;
uniform lowp float bloom_mult;

V_FOG_UNIFORMS

//current uniform total: 85 vectors, 42 remain
// cost of 4 more bones 16 + 12 = 28

void main()
{
    
    int b = int(bone.x)-1;
    
    vec3 n = normalize(normal);
    vec3 t = normalize(binormal.xyz);
    vec3 bt = -normalize(cross(n, t))*binormal.w;
    
    vec4 vert_pos = MAKE_MAT4_FROM_ARRAY(mMatrices, b) * position;
    vec4 world_position = mMatrix * vert_pos;
    gl_Position = vpMatrix * world_position;
    
    frag_uv1 = uv1;
    
    
    mat3 tsp = mat3(t.x, t.y, t.z,
                    bt.x, bt.y, bt.z,
                    n.x, n.y, n.z);
    
    
    tsp = normalMatrix * MAKE_MAT3_FROM_ARRAY(minvMatrices, b) * tsp;
    STORE_MAT3_VARYING(tangentSpace,
                       tsp[0][0], tsp[0][1], tsp[0][2],
                       tsp[1][0], tsp[1][1], tsp[1][2],
                       tsp[2][0], tsp[2][1], tsp[2][2])
    
    vec3 to_light = vec3(0, 1.4, 0.85) - vert_pos.xyz;
    v_bloom_dir = max(0.0, 1.0 - length(to_light) / 0.9) * bloom_mult * (normalMatrix * normalize(to_light));
    
    v_fog = PARAM_FOG(fog_mult);
    v_shadow_coord = (shadow_projection * world_position);
    
}
